/**
 * Copyright Copyright 2008-14 Simon Andrews
 *
 *    This file is part of FocalPoint.
 *
 *    FocalPoint is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FocalPoint is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FocalPoint.GUI.Thumbnail;

import gov.nih.info.rsb.imagej.JImagePanel;
import ij.ImagePlus;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.Thumbnail.Resizer.ImageResizer;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.Thumbnail.Resizer.ResizeListener;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.events.ImageLoadingListener;
import uk.ac.bbsrc.babraham.FocalPoint.Preferences.Preferences;

public abstract class ThumbnailPanel extends JPanel implements ResizeListener, ChangeListener, Runnable, MouseListener {

	// This is a generic abstract class which will be subclassed
	// for the different types of image we can display

	// This is the default width.  The height is dynamic based
	// on the aspect ratio of the image.

	protected FileDetailsPanel fileDetails;

	public static int thumbnailWidth = Preferences.getInstance().iconWidth();
	protected int width;
	protected int height;
	protected int slices;
	protected int channels;

	protected boolean deferLoading = false;
	protected JPanel notLoadedPanel;
	protected JLabel notLoadedLabel;

	protected boolean imageBroken = false;

	protected int lastWidth = 0;

	private JImagePanel canvas = null;
	protected ImagePlus [] resizedIPs = null;

	private JSlider slicesSlider = null;
	private JSlider channelsSlider = null;

	protected ImageLoadingListener imageLoadingListener = null;

	protected JPopupMenu popupMenu = null;

	protected int selectedChannel;

	abstract ImagePlus [] loadImages (File file) throws IOException;
	abstract boolean shouldDeferLoading (File file);
	
	/**
	 * 
	 * @param file
	 * @return
	 * 
	 * This call is needed because some file types use more than one
	 * file to store the image, so the size we want to see is the
	 * total data files for the image, not just the size of the 
	 * actual file.  Eg Olympus files put their data in a folder
	 * which must be moved around with the .oif index file.
	 */
	abstract long getFileSize(File file);
	abstract boolean canReadFile(File file);

	
	public ThumbnailPanel (FileDetailsPanel fileDetails) throws IncompatibleFileTypeException {
		this.fileDetails = fileDetails;
		if (!canReadFile(fileDetails.file())) {
			throw new IncompatibleFileTypeException();
		}
		setLayout(new BorderLayout());

		if (isImage()) {
			notLoadedPanel = new JPanel();
			notLoadedLabel = new JLabel("Loading...");
			notLoadedPanel.setPreferredSize(new Dimension(thumbnailWidth,getHeight()));
			notLoadedPanel.add(notLoadedLabel);
			add(notLoadedPanel,BorderLayout.CENTER);
			
			if (Preferences.getInstance().deferLargeImages() && shouldDeferLoading(fileDetails.file())) {
				deferLoading();
			}
		}

		updateThumbnail();
	}
	
	public void cleanUp () {
		/*
		 * Called when we're being destroyed.  Just blank
		 * whatever we're storing to try to help the GC
		 * along.
		 */
		
		removeAll();
		resizedIPs = null;
		canvas = null;
		imageLoadingListener = null;
		popupMenu = null;
		fileDetails = null;
	}

	public void setImageLoadingListener (ImageLoadingListener ill) {
		imageLoadingListener = ill;
	}

	protected void deferLoading () {
		deferLoading = true;

		notLoadedPanel = new JPanel();
		notLoadedLabel.setText("Click to Load...");
		remove(notLoadedPanel);
		notLoadedPanel.addMouseListener(new MouseListener() {

			public void mouseReleased(MouseEvent e) {}

			public void mousePressed(MouseEvent e) {}

			public void mouseExited(MouseEvent e) {}

			public void mouseEntered(MouseEvent e) {}

			public void mouseClicked(MouseEvent e) {
				deferLoading = false;
				notLoadedLabel.setText("Loading...");
				try {
					updateThumbnail();
				}
				catch (Exception e1) {
					e1.printStackTrace();
				}
			}

		});
		notLoadedPanel.add(notLoadedLabel);
		add(notLoadedPanel,BorderLayout.CENTER);
		revalidate();

	}


	public static void setThumbnailWidth (int width) {
		thumbnailWidth = width;
	}

	public void updateThumbnail (){

		if (deferLoading || imageBroken) {
			notLoadedPanel.setPreferredSize(new Dimension(thumbnailWidth,getHeight()));
			notLoadedPanel.revalidate();
			return;
		}


		Thread t = new Thread(this);
		t.start();

	}

	abstract boolean isImage();

	protected int width () {
		return width;
	}

	protected int height() {
		return height;
	}

	protected int slices () {
		return slices;

	}

	protected int channels () {
		return channels;
	}

	public void ImageResized(ImagePlus[] ip) {
		if (ip != null) {
			lastWidth = thumbnailWidth;
			resizedIPs = ip;
		}

		if (notLoadedPanel != null) {
			remove(notLoadedPanel);
			notLoadedPanel = null;
		}

		// Check if we need to add any decoration

		//TODO:  On linux the sliders look odd because the LAF puts a label
		// over the top of them.  I need to find out how to remove this.
		
		if (channels > 1 && channelsSlider == null) {
			channelsSlider = new JSlider(JSlider.HORIZONTAL,0,channels-1,0);
			channelsSlider.addChangeListener(this);
			channelsSlider.setOpaque(true);
			channelsSlider.setPreferredSize(new Dimension(thumbnailWidth,20));
			channelsSlider.setBackground(new Color(192,128,128));
			add(channelsSlider,BorderLayout.NORTH);
		}
		
		boolean madeNewSlicesSlider = false;
		if (slices>1 && slicesSlider == null) {
			madeNewSlicesSlider = true;
			slicesSlider = new JSlider(JSlider.HORIZONTAL,0,slices-1,0);
			slicesSlider.addChangeListener(this);
			slicesSlider.setOpaque(true);
			slicesSlider.setPreferredSize(new Dimension(thumbnailWidth,20));
			slicesSlider.setBackground(new Color(128,128,192));
			add(slicesSlider,BorderLayout.SOUTH);
			
		}

		if (canvas != null) {
			remove(canvas);
		}

		if (slicesSlider != null) {
			slicesSlider.setPreferredSize(new Dimension(thumbnailWidth,20));
			resizedIPs[selectedChannel].setSlice(slicesSlider.getValue()+1);
		}

		if (channelsSlider != null) {
			channelsSlider.setPreferredSize(new Dimension(thumbnailWidth,20));
		}

		canvas = new JImagePanel(resizedIPs[selectedChannel]);
		canvas.addMouseListener(this);
		add(canvas,BorderLayout.CENTER);
		
		if (madeNewSlicesSlider) {
			// We initially set the slices slider to the middle position
			slicesSlider.getModel().setValue((slices-1)/2);
		}

		
		revalidate();
		canvas.repaint();

	}

	public void stateChanged(ChangeEvent ce) {
		// Someone has dragged a slider.  We need
		// to update the image being shown

		if (ce.getSource() == slicesSlider) {
			int frame = ((JSlider)ce.getSource()).getValue();

			resizedIPs[selectedChannel].setSlice(frame+1);

			canvas.setImageUpdated();
			canvas.repaint();
		}

		else if (ce.getSource() == channelsSlider) {
			selectedChannel = ((JSlider)ce.getSource()).getValue();
			updateThumbnail();
		}
	}

	

	public void run () {

		try {
			Thread.sleep(20);
		} 
		catch (InterruptedException e1) {
			System.err.println("Interrupted");
		}

		if (thumbnailWidth > lastWidth || lastWidth == 0) {
			if (imageLoadingListener != null)
				imageLoadingListener.ImageLoading();
			try {
				ImagePlus [] ip = loadImages(fileDetails.file());
				channels = ip.length;

				width = ip[selectedChannel].getWidth();
				height = ip[selectedChannel].getHeight();
				slices = ip[selectedChannel].getStackSize();

				// If we've changed directories whilst the image was
				// loading then fileDetails might now be null. This
				// means we can stop right here.
				if (fileDetails == null) return;
				
				
				fileDetails.addImageDetails(width, height, slices, channels);

				ImageResizer.getInstance().resizeImages(ip, this);
			}
			catch (Exception e) {
				e.printStackTrace();
				imageBroken = true;
				notLoadedLabel.setText("Failed to Load");
			}
			imageLoadingListener.ImageLoaded();
		}
		else if (thumbnailWidth == lastWidth) {
			// We can skip straight to redrawing the current image
			ImageResized(resizedIPs);			
		}
		else {
			// We need to resize, but we can use the current
			// imageplus objects to start from
			ImageResizer.getInstance().resizeImages(resizedIPs, this);
		}		
	}

	public void mouseClicked(MouseEvent me) {}

	public void mouseEntered(MouseEvent arg0) {}

	public void mouseExited(MouseEvent arg0) {
	}

	public void mousePressed(MouseEvent me) {

		if (popupMenu == null) {
			popupMenu = new ThumbnailPopupMenu(fileDetails,isImage());
		}
		popupMenu.show(canvas,me.getX(), me.getY());
	}

	public void mouseReleased(MouseEvent arg0) {
	}


}

