/**
 * Copyright Copyright 2008-14 Simon Andrews
 *
 *    This file is part of FocalPoint.
 *
 *    FocalPoint is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FocalPoint is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FocalPoint.GUI;

import ij.IJ;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.File;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import uk.ac.bbsrc.babraham.FocalPoint.FocalPointApplication;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.Thumbnail.FilePanel;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.Thumbnail.FilesOnlyFileFilter;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.Thumbnail.NotAnImageThumbnailPanel;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.Thumbnail.ThumbnailPanel;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.events.DirectorySelectionListener;
import uk.ac.bbsrc.babraham.FocalPoint.GUI.events.IconSizeListener;
import uk.ac.bbsrc.babraham.FocalPoint.Preferences.Preferences;

public class ThumbnailCollection extends JPanel implements DirectorySelectionListener, IconSizeListener, Runnable{

	// This class is a container for individual Panels for each image.  It
	// listens for changes from the directory tree and then updates its view
	
	private Vector<JPanel> fileComponents = new Vector<JPanel>();
	private File [] dirContents = new File [0];
	private JPanel innerPanel;
	private JScrollPane scrollPane;
	private GridBagConstraints cons;
	private boolean finishedLastLoad = true;
		
	
	private boolean updating = false;
	private boolean stopNow = false;
	
	public ThumbnailCollection () {
		setLayout(new BorderLayout());
		innerPanel = new JPanel();
		innerPanel.setLayout(new GridBagLayout());
		
		scrollPane = new JScrollPane(innerPanel);
		scrollPane.getVerticalScrollBar().setUnitIncrement(5);
		AboutPanel ap = new AboutPanel();
		fileComponents.add(ap);
		innerPanel.add(ap);
		cons = new GridBagConstraints();
		cons.fill = GridBagConstraints.HORIZONTAL;
		cons.anchor = GridBagConstraints.NORTH;
		cons.weightx=0.5;
		cons.weighty=0.5;
		cons.gridx=1;
		
		add(scrollPane,BorderLayout.CENTER);
	}
		
	public synchronized void directorySelected(File directory) {
		
		// Check if we need to interrupt a previous update
		if (updating) {
			stopNow = true;
			while (updating) {
				try {
					Thread.sleep(20);
				} 
				catch (InterruptedException e) {}
			}
		}
		
		if (directory == null) {
			dirContents = null;
		}
		else {
			dirContents = directory.listFiles(FilesOnlyFileFilter.getInstance());
			if (dirContents != null) {
				Arrays.sort(dirContents);
			}
		}
		
		// Remove any existing objects
		innerPanel.removeAll();
		
		revalidate();
		scrollPane.revalidate();
		innerPanel.revalidate();
		repaint();
		scrollPane.repaint();
		innerPanel.repaint();
		
		Enumeration<JPanel> e = fileComponents.elements();
		while (e.hasMoreElements()) {
			JPanel o = e.nextElement();
			innerPanel.removeAll();
			if (o instanceof FilePanel) {
				((FilePanel)o).cleanUp();				
			}
			else {
				System.err.println("not cleaning up this time...");
			}
		}
		
		fileComponents.removeAllElements();
		
		System.gc();
		IJ.freeMemory();
		System.gc();
		
		
		if (dirContents != null) {
			Thread t = new Thread(this);
			updating = true;
			finishedLastLoad = false;
			t.start();
		}
	}

	public void run() {
		
		// We're either making up new file panels, or we're updating
		// existing ones.
				
		// If there are things in the fileComponents vector then we're
		// updating, otherwise we're starting anew.
		
		// We may never had selected a directory, in which case dirContents
		// will be null
		
		if (finishedLastLoad) {
			Enumeration<JPanel> e = fileComponents.elements();
			while (e.hasMoreElements()) {
				if (stopNow) {
					stopNow = false;
					updating = false;
					return;
				}
				ThumbnailPanel tp = ((FilePanel)e.nextElement()).getThumbnailPanel();
				if (tp != null) {
					tp.updateThumbnail();
				}
			}			
		}
		
		else {
			// We need to take the contents of dirContents and make up
			// individual File Panels for each one.

			/* There is a problem when loading big files that it
			 * 
			 *  takes too long to interrupt the loading.  In these cases
		`	 *  it would be nice to have some way to interrupt the loading
             *  to get a quicker response.  Not sure that's going to be
             *  possible though...
			 */
			
			
			/*
			 * On Java <=1.5 there is a ridiculous bug where you can't
			 * put more than 512 objects into a GridBagLayout.  This
			 * means the program breaks when there are more than 512
			 * files in a directory.  To get around this we will add
			 * files in groups of 500, then we will put in another
			 * nested JPanel in which we can add another 500.  We
			 * can repeat this no matter how many files we have to
			 * deal with.  We need to adjust the weights to make sure
			 * the last inserted panel gets much more weight than the
			 * rest of the inserted filepanels.
			 */
			
			JPanel currentPanel = innerPanel;
			int currentCount = 0;
			
			cons.weighty = 0.00001;
			for (int f=0;f<dirContents.length;f++) {
				if (stopNow) {
					stopNow = false;
					updating = false;
					return;
				}
				FilePanel p = new FilePanel(dirContents[f]);
				cons.gridy=currentCount;
				
				
				// We might be hiding non-image panels.
				if (Preferences.getInstance().showOnlyImages() && p.getThumbnailPanel() instanceof NotAnImageThumbnailPanel) {
					continue;
				}
				else {
					currentPanel.add(p,cons);
					fileComponents.add(p);
					p.setIndex(fileComponents.size());
				}
				
				currentPanel.revalidate();
				innerPanel.revalidate();
				scrollPane.validate();
				revalidate();
				repaint();
				innerPanel.repaint();
				scrollPane.repaint();
				currentPanel.repaint();
				++currentCount;
				if (currentCount > 500) {
					JPanel nextPanel = new JPanel();
					nextPanel.setLayout(new GridBagLayout());
					cons.weighty = 0.99999;
					cons.gridy = currentCount;
					currentPanel.add(nextPanel,cons);
					currentPanel = nextPanel;
					currentCount = 0;
					cons.weighty = 0.00001;
				}
				
				try {
					Thread.sleep(10);
				}
				catch (InterruptedException ex) {}
			}
			cons.weighty=0.9999;
			cons.gridy++;
			innerPanel.add(new JPanel(),cons);
			innerPanel.revalidate();
			scrollPane.validate();
			revalidate();
			repaint();
			innerPanel.repaint();
			scrollPane.repaint();
		}
		finishedLastLoad = true;
		updating = false;
	}

	public void setNewIconSize(int width) {
		ThumbnailPanel.setThumbnailWidth(width);
		
		if (updating) {
			stopNow = true;
			
			while (updating) {
				try {
					Thread.sleep(20);
				} 
				catch (InterruptedException e) {}
			}
		}
		
		Thread t = new Thread(this);
		updating = true;
		t.start();
	}
	
	private class AboutPanel extends JPanel {
		public AboutPanel () {
			
			setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.gridx=1;
			c.gridy=0;
			c.weightx=0.5;
			c.weighty=0.5;
			c.fill=GridBagConstraints.HORIZONTAL;
			
			add(new JLabel(new ImageIcon(ClassLoader.getSystemResource("Resources/focal_point_logo.png")),JLabel.CENTER),c);
			c.gridy++;
			add(new JLabel("FocalPoint Image Viewer",JLabel.CENTER),c);
			c.gridy++;
			add(new JLabel("Version "+FocalPointApplication.version,JLabel.CENTER),c);
			c.gridy++;
			add(new JLabel("http://www.bioinformatics.babraham.ac.uk/projects/",JLabel.CENTER),c);
			c.gridy++;
			add(new JLabel(" ",JLabel.CENTER),c);
			c.gridy++;
			add(new JLabel("\u00a9 Simon Andrews, Babraham Bioinformatics, 2008-14",JLabel.CENTER),c);
			c.gridy++;
			add(new JLabel(" ",JLabel.CENTER),c);
			c.gridy++;
			add(new JLabel("Additional code from:",JLabel.CENTER),c);
			c.gridy++;
			add(new JLabel(" ",JLabel.CENTER),c);
			c.gridy++;
			add(new JLabel("ImageJ - http://rsb.info.nih.gov/ij/",JLabel.CENTER),c);
			c.gridy++;
			add(new JLabel("BioFormats - http://www.loci.wisc.edu/ome/formats.html",JLabel.CENTER),c);
			c.gridy++;
			add(new JLabel("LSM Toolbox - http://imagejdocu.tudor.lu/Members/ppirrotte/lsmtoolbox/",JLabel.CENTER),c);
	
		}
	}
}
