/**
 * Copyright Copyright 2008-14 Simon Andrews
 *
 *    This file is part of FocalPoint.
 *
 *    FocalPoint is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    FocalPoint is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with StackMeasure; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.FocalPoint.Preferences;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;

public class Preferences {

	/*
	 * This class stores the preferences for the program
	 * and is responsible for loading, saving and updating
	 * them.
	 */
	
	private static Preferences prefs = new Preferences();
	private File preferencesFile;
	
	/*
	 * These are the actual preferences we store
	 */
	
	private boolean showOnlyImages = true;
	private boolean deferLargeImages = true;
	private int iconWidth = 150;
	private File pathToImageJ = null;
	
	private Preferences () {

		preferencesFile= new File(System.getProperty("user.home")+"/.focalpoint");

		setDefaults();
		try {
			loadPreferences();
		}
		catch (IOException e) {
			System.err.println("Error loading preferences: "+e.getMessage());
		}
	}
	
	private void loadPreferences () throws IOException {
		if (! preferencesFile.exists()) return;
		
		String line;
		
		BufferedReader br = new BufferedReader(new FileReader(preferencesFile));
		
		while ((line = br.readLine())!=null) {
			String [] parts = line.split("\t");
			
			if (parts[0].equals("showOnlyImages")) {
				// The default is true
				if (parts[1].equals("false")) {
					setShowOnlyImages(false);
				}
			}

			else if (parts[0].equals("deferLargeImages")) {
				// The default is true
				if (parts[1].equals("false")) {
					setDeferLargeImages(false);
				}
			}
			
			else if (parts[0].equals("iconWidth")) {
				setIconWidth(Integer.parseInt(parts[1]));
			}
			
			else if (parts[0].equals("pathToImageJ")) {
				File ij = new File(parts[1]);
				setPathToImageJ(ij);
			}
			
			else {
				System.err.println("Unknown preference '"+parts[0]+"' found in file");
			}
		}
		
		br.close();
		
	}
	
	public void savePreferences () throws IOException {
		PrintWriter p = new PrintWriter(preferencesFile);
		
		p.println("showOnlyImages\t"+showOnlyImages);
		p.println("deferLargeImages\t"+deferLargeImages);
		p.println("iconWidth\t"+iconWidth);
		
		if (pathToImageJ != null) {
			p.println("pathToImageJ\t"+pathToImageJ.getAbsolutePath());
		}
		
		p.close();
		
	}
	
	public void resetPreferences () {
		setShowOnlyImages(true);
		setDeferLargeImages(true);
		setIconWidth(150);
		pathToImageJ = null;
	}
	
	private void setDefaults() {
		showOnlyImages = true;
		deferLargeImages = true;
		iconWidth = 150;
		pathToImageJ = null;
	}
	
	public static Preferences getInstance () {
		return prefs;
	}
	
	public boolean showOnlyImages () {
		return showOnlyImages;
	}
	
	public void setShowOnlyImages (boolean b) {
		showOnlyImages = b;
	}
	
	public boolean deferLargeImages () {
		return deferLargeImages;
	}
	
	public void setDeferLargeImages (boolean b) {
		deferLargeImages = b;
	}
	
	public int iconWidth () {
		return iconWidth;
	}
	
	public void setIconWidth (int w) {
		if (w>500) w=500;
		if (w<5) w=5;
		iconWidth = w;
	}
	
	public File pathToImageJ () {
		return pathToImageJ;
	}
	
	public void setPathToImageJ (File f) throws IOException {
		if (f == null) {
			pathToImageJ = null;
			return;
		}
		
		if (!f.exists()) {
			throw new IOException("ImageJ file '"+f.getAbsolutePath()+"' does not exist");
		}
			
		if (!f.isFile()) {
			throw new IOException("ImageJ file '"+f.getAbsolutePath()+"' is not a file");
		}
		
		if (!f.canRead()) {
			throw new IOException("ImageJ file '"+f.getAbsolutePath()+"' is not readable");
		}

		pathToImageJ = f;
	}
}
